#include "includes.hpp"

x_nines::x_nines() {
	this->g_counter=0;
	memset(this->hash,0,sizeof(this->hash));
	memset(this->surplus,0,sizeof(this->surplus));
}

unsigned int x_nines::getCycles() {
	return this->cycles;
}

void x_nines::reset_surplus(int num) {
	int i;
	for (i=0;i<num;i++) {
		switch (i%3) {
			case 1:
				break;
			case 2:
				this->surplus[i]=0;
				break;
			case 0:
				this->surplus[i]=-1;
				break;
		}
	}
}

int x_nines::inc_surplus(int pos) {
	int rc=0;
	if (pos>_SURPLUS_SIZE) {return rc;}
	switch (pos%3) {
		case 2:
			this->surplus[pos]++;
			rc=1;
			if (this->surplus[pos]>3) {
				this->reset_surplus(pos+1);
				rc=this->inc_surplus(pos+1);
			}
			break;
		case 1:
			rc=this->inc_surplus(pos+1);
			break;
		case 0:
			if (this->surplus[pos]<0) {
				this->surplus[pos]=4;
				rc=1;
			} else {
				this->surplus[pos]++;
				rc=1;
				if (this->surplus[pos]>5) {
					this->reset_surplus(pos+1);
					rc=this->inc_surplus(pos+1);
				}
			}
			break;
	}
	return rc;
}

int x_nines::getMin() {
	for (int i=1;i<_HASH_SIZE;i++) {
		if (!this->hash[i]) {
			return i;
		}
	}
	return 0;
}

void x_nines::run() {
	int i,j;
	float num;
	this->reset_surplus(_SURPLUS_SIZE);
	this->cycles=0;
	do {
		j=0;
		for (i=0;i<_SURPLUS_SIZE;i+=3) {
			switch (this->surplus[i]) {
				case _SIGN_O_BRACKET:
					j++;
					break;
				case _SIGN_C_BRACKET:
					j--;
					break;
			}
			if (j<0) {break;}
		}
		if (j>=0) {
			this->g_counter=0;
			this->p_counter=j;
			this->l_counter=-1;
			this->reset();
			this->convert();
			num=this->calc();
			if ((num>0) && (floor(num)==num)) {
				if (num<_HASH_SIZE) {
					this->hash[int(num)]=1;
				}
			}
			if (_SEARCH_FOR && (num==_SEARCH_FOR)) {
				i=this->cycles;
				this->cycles=_DUMP_EACH;				
				this->g_counter=0;
				this->p_counter=j;
				this->l_counter=-1;
				this->reset();
				this->convert();
				printf("=%i\n",int(num));
				this->cycles=i;
				return;
			}
			if (!(this->cycles%_DUMP_EACH)) {
				printf("=%i   [%i]\n",int(num),this->getMin());
			}
			this->cycles++;
		}
	} while (this->inc_surplus());
}

int x_nines::getNextEntity(int *num,int *type) {
	int flag;
	if (this->g_counter) {
		(*num)=_SIGN_C_BRACKET;
		(*type)=_TP_SIGN;
		if (!(this->cycles%_DUMP_EACH)) {
			this->dumpEntity((*num),(*type));
		}
		this->g_counter--;
		return 1;
	}
	this->l_counter++;	
	if (this->l_counter>_SURPLUS_SIZE) {
		if (this->p_counter) {
			(*num)=_SIGN_C_BRACKET;
			(*type)=_TP_SIGN;
			if (!(this->cycles%_DUMP_EACH)) {
				this->dumpEntity((*num),(*type));
			}
			this->p_counter--;
			return 1;
		}
		return 0;
	} else {
		do {
			flag=0;
			if ((this->l_counter%3)==1) {
				(*num)=9;
				(*type)=_TP_NUMBER;
			} else {
				(*type)=_TP_SIGN;
				if (this->surplus[this->l_counter]>=0) {
					(*num)=this->surplus[this->l_counter];
					if ((*num)==_SIGN_C_BRACKET) {
						this->g_counter++;
						this->l_counter++;
						flag=1;
					}
				} else {
					this->l_counter++;
					flag=1;
				}
			}
		} while (flag);
		if (!(this->cycles%_DUMP_EACH)) {		
			this->dumpEntity((*num),(*type));
		}
		return 1;
	}
}

int main() {
	clock_t start, finish;
	x_nines *calc;
	double elapsed_time;
	calc=new x_nines();
	if (_SEARCH_FOR) {
		printf("Searching expression for: %i\n",(int)_SEARCH_FOR);
	} else {
		printf("Searching for minimum integer\n");
	}
	start=clock();
	calc->run();
	finish=clock();
	if (!_SEARCH_FOR) {
		printf("Found min: %i\n",calc->getMin());
	}
	free(calc);
	elapsed_time=(double)(finish - start);
	printf("Time taken: %2.2f second(s)\n",elapsed_time/CLOCKS_PER_SEC);
	if (elapsed_time>0) {
		printf( "Speed: %1.0f cycles per second\n",floor(calc->getCycles()/elapsed_time*CLOCKS_PER_SEC));
	} else {
		printf( "Speed: n/a\n");
	}
	return 0;
}
