#include "includes.hpp"

b_calc::b_calc() {
	this->s_stack=new stack();
	this->s_out=new stack(sizeof(_OUT_BUF));
	this->s_calc=new stack(sizeof(float));	
}
b_calc::~b_calc() {
	free(this->s_stack);
	free(this->s_out);
	free(this->s_calc);	
}

void b_calc::reset() {
	this->s_out->reset();
	this->s_stack->reset();	
}


int b_calc::getSignPriority(int sign) {
	switch (sign) {
		case _SIGN_O_BRACKET:
			return 1;
		case _SIGN_PLUS:
		case _SIGN_MINUS:
			return 2;
		case _SIGN_MUL:
		case _SIGN_DIV:
			return 3;
		default:
			return 0;
	}
}

void b_calc::convert() {
	_OUT_BUF tmp;
	int num,type;
	while (this->getNextEntity(&num,&type)) {
		if (type==_TP_NUMBER) {
			tmp.num=num;
			tmp.type=type;
			this->s_out->push(&tmp);
		} else {
			tmp.type=_TP_SIGN;
			switch (num) {
				case _SIGN_O_BRACKET:
					this->s_stack->push(&num);
					break;
				case _SIGN_C_BRACKET:
					if (this->s_stack->count()) {
						do {
							this->s_stack->pop(&tmp.num);
							if (tmp.num!=_SIGN_O_BRACKET) {
								this->s_out->push(&tmp);
							}
						} while ((tmp.num!=_SIGN_O_BRACKET) && (this->s_stack->count()));
					}
					break;
				default:
					while (this->s_stack->count()) {
						this->s_stack->get(&tmp.num);
						if (this->getSignPriority(tmp.num)<this->getSignPriority(num)) {break;}
						this->s_out->push(&tmp);
						this->s_stack->dec();
					}
					this->s_stack->push(&num);
			}
		}
	}
	tmp.type=_TP_SIGN;
	while (this->s_stack->count()) {
		this->s_stack->pop(&tmp.num);
		this->s_out->push(&tmp);
	}
}

float b_calc::calc() {
	_OUT_BUF tmp;
	int i=0;
	float rc=0;
	float a,b;
	this->s_calc->reset();
	while (i<this->s_out->count()) {
		this->s_out->next(&tmp,&i);
		if (tmp.type==_TP_NUMBER) {
			a=(float)tmp.num;
			this->s_calc->push(&a);
		} else {
			a=0;b=0;
			if (this->s_calc->count()) {
				this->s_calc->pop(&a);
			}
			if (this->s_calc->count()) {
				this->s_calc->pop(&b);
			}
			switch (tmp.num) {
				case _SIGN_PLUS:
					a=b+a;
					break;
				case _SIGN_MINUS:
					a=b-a;
					break;
				case _SIGN_MUL:
					a=b*a;
					break;
				case _SIGN_DIV:
					//need div by zero handling if your compiler doesn't do this
					a=b/a;
					break;		
			}
			this->s_calc->push(&a);
		}
	}
	if (this->s_calc->count()) {
		this->s_calc->pop(&rc);
	}
	return rc;
}


void b_calc::dumpStack() {
	int i=0;
	int sign;
	printf("Stack: ");
	while (i<this->s_stack->count()) {
		this->s_stack->next(&sign,&i);
		this->dumpEntity(sign,_TP_SIGN);
	}
	printf("\r\n");
}

void b_calc::dumpEntity(int num,int type) {
	if (type==_TP_NUMBER) {
		printf("%i",num);
	} else {
		switch (num) {
			case _SIGN_PLUS:
				printf("+");
				break;
			case _SIGN_MINUS:
				printf("-");
				break;
			case _SIGN_MUL:
				printf("*");
				break;
			case _SIGN_DIV:
				printf("/");
				break;
			case _SIGN_O_BRACKET:
				printf("(");
				break;
			case _SIGN_C_BRACKET:				
				printf(")");
				break;
		}
	}
}

void b_calc::dump() {
	_OUT_BUF tmp;
	int i=0;
	printf("Result: ");	
	while (i<this->s_out->count()) {
		this->s_out->next(&tmp,&i);
		this->dumpEntity(tmp.num,tmp.type);
	}
	printf("\n");
}
